@@ -9,7 +9,6 @@
 
 struct xfs_inode;
 struct xfs_buftarg;
-struct xfs_dir_ops;
 struct xfs_da_geometry;
 
 /*
@@ -87,8 +86,6 @@ typedef struct xfs_mount {
 
 	struct xfs_da_geometry	*m_dir_geo;	/* directory block geometry */
 	struct xfs_da_geometry	*m_attr_geo;	/* attribute block geometry */
-	const struct xfs_dir_ops *m_dir_inode_ops; /* vector of dir inode ops */
-	const struct xfs_dir_ops *m_nondir_inode_ops; /* !dir inode ops */
 
 	/*
 	 * anonymous struct to allow xfs_dquot_buf.c to compile.
